import click
import sys
import yaml

from defrost import PipFreeze

exit_codes = {
    'error': {'hard': 1, 'normal': 1, 'soft': 0},
    'warn': {'hard': 1, 'normal': 0, 'soft': 0},
}


def _parse_yaml(requirement_file):
    try:
        return yaml.load(requirement_file)
    except yaml.YAMLError as err:
        click.echo("YAMLError: {}".format(err), err=True)
        sys.exit(1)


@click.command()
@click.option(
    '-x', '--exit-mode', default='normal',
    type=click.Choice(['soft', 'normal', 'hard']),
    help="""\
soft: always return exit code 0; normal (default): return exit code 1 on \
"error"; hard: return exit code 1 on "warn" or "error".""")
@click.argument('requirement_file', type=click.File())
@click.argument('pip_freeze_file', type=click.File())
def defrost(exit_mode, requirement_file, pip_freeze_file):
    """
    Check if the output of pip freeze satisfies the YAML requirement file. The
    pip freeze output may be passed as stdin:

        pip freeze | defrost reqs.yml -

    """
    reqs = _parse_yaml(requirement_file)
    pip_freeze = PipFreeze(pip_freeze_file.read())
    pip_freeze.load_requirements(reqs)

    exit_code = 0
    for package in pip_freeze.deprecated:
        click.echo("{}: {!r} does not satisfy {!r}: {}".format(
            package.deprecation_severity,
            package,
            package.deprecated_by,
            package.deprecation_reason,
        ), err=True)
        if exit_code == 0:
            exit_code = exit_codes[package.deprecation_severity][exit_mode]

    sys.exit(exit_code)


@click.command()
@click.argument('requirement_file', type=click.File())
def lint(requirement_file):
    """
    Validate a YAML requirement file.
    """
    from defrost.validation import validate
    reqs = _parse_yaml(requirement_file)

    try:
        validate(reqs)
        sys.exit(0)
    except Exception as err:
        click.echo(err, err=True)
        sys.exit(1)



import io

from collections import OrderedDict
from .package import Package, Requirement


def _parse_pip_freeze(pip_freeze_output):
    if isinstance(pip_freeze_output, bytes):
        pip_freeze_output = pip_freeze_output.decode('utf-8')

    pip_freeze_output = io.StringIO(pip_freeze_output)

    for req in pip_freeze_output.readlines():
        req = req.strip()
        if not req or req.startswith(('-', '#')):
            continue
        yield Package(req)


def _to_requirement(requirement):
    if isinstance(requirement, Package):
        requirement = Requirement(str(requirement))

    elif isinstance(requirement, str):
        requirement = Requirement(requirement)

    return requirement


class PipFreeze(object):
    def __init__(self, pip_freeze_output):
        self._load_pip_freeze(pip_freeze_output)
        self.deprecated = []

    def _load_pip_freeze(self, pip_freeze_output):
        self._packages = OrderedDict()

        packages = _parse_pip_freeze(pip_freeze_output)
        for package in packages:
            self._packages[package.id] = package

    def __contains__(self, requirement):
        requirement = _to_requirement(requirement)
        package = self._packages.get(requirement.id)
        return package in requirement

    def __getitem__(self, package_name):
        requirement = _to_requirement(package_name)
        package = self._packages[requirement.id]
        return package

    def get(self, requirement, default=None):
        try:
            return self[requirement]
        except KeyError:
            return default

    def __iter__(self):
        for package in self._packages.values():
            yield package

    def __len__(self):
        return len(self._packages)

    def __nonzero__(self):
        # Python 2.x
        return self.__bool__()  # pragma: no cover

    def __bool__(self):
        # Python 3.x
        return bool(self._packages)

    def load_requirements(self, requirements):
        for req in requirements['requirements']:
            requirement = Requirement(req['requirement'])
            package = self._packages.get(requirement.id)
            if package is None:
                continue
            if package not in requirement:
                deprecate_kwargs = {'deprecated_by': requirement}
                if 'reason' in req:
                    deprecate_kwargs['reason'] = req['reason']
                if 'severity' in req:
                    deprecate_kwargs['severity'] = req['severity']
                package.deprecate(**deprecate_kwargs)
                self.deprecated.append(package)



import pkg_resources


class Requirement(object):
    """
    A package requirement.

    A Requirement object represents one or more versions for a given package.

    For example::

        foo==1.0
        foo<2.0
        foo>=3.0,<4.0

    """
    def __init__(self, requirement):
        """
        ``requirement`` is a string that represents a package requirement.

        """
        self._req = pkg_resources.Requirement.parse(requirement)

        self.id = self._req.key
        self.name = self._req.project_name
        self.specifiers = self._req.specs

    def __contains__(self, package):
        if not isinstance(package, Package):
            return False

        if self.id != package.id:
            return False

        return package.version in self._req

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False

        return self._req == other._req

    def __hash__(self):
        return self._req.__hash__()

    def __ne__(self, other):
        return not self == other

    def __repr__(self):
        return "Requirement({})".format(self)

    def __str__(self):
        return str(self._req)


class Package(object):
    """
    An exact package version.

    A Package object represents exactly one version of a package.

    For example::

        foo==1.0
        bar==2.3

    """
    def __init__(self, pinned_requirement):
        self._req = Requirement(pinned_requirement)
        self.deprecated = False
        self.deprecation_reason = None
        self.deprecated_by = None
        self.deprecation_severity = None

        # Ensure we're dealing with an exact package version
        if len(self._req.specifiers) != 1 or \
                self._req.specifiers[0][0] not in ('==', '==='):
            raise ValueError(
                '{} does not represent an exact package version; '
                'the format should be foo==1.0'.format(pinned_requirement)
            )

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False

        return (self.name, self.version) == (other.name, other.version)

    def __hash__(self):
        return self._req.__hash__()

    def __lt__(self, other):
        return (self.name.lower(), self.version) < \
               (other.name.lower(), other.version)

    def __ne__(self, other):
        return not self == other

    def __repr__(self):
        return "Package({})".format(self)

    def __str__(self):
        return str(self._req)

    @property
    def id(self):
        return self._req.id

    @property
    def name(self):
        return self._req.name

    @property
    def version(self):
        return self._req.specifiers[0][1]

    def deprecate(self, reason=None, deprecated_by=None, severity='error'):
        self.deprecated = True
        self.deprecation_reason = reason
        self.deprecated_by = deprecated_by
        self.deprecation_severity = severity



import pytest
from defrost import Package


@pytest.mark.parametrize("req, req_name, req_specs, req_id, req_version, req_repr, req_str", [
    ('foobar==1.2', 'foobar', [('==', '1.2')], 'foobar', '1.2', 'Package(foobar==1.2)', 'foobar==1.2'),
    ('foobar==1.2\n', 'foobar', [('==', '1.2')], 'foobar', '1.2', 'Package(foobar==1.2)', 'foobar==1.2'),
    ('foobar===1.2', 'foobar', [('===', '1.2')], 'foobar', '1.2', 'Package(foobar===1.2)', 'foobar===1.2'),
])
def test_package(req, req_name, req_specs, req_id, req_version, req_repr, req_str):
    from defrost import Package
    package = Package(req)
    assert package.id == req_id
    assert package.name == req_name
    assert package.version == req_version
    assert package.deprecated is False
    assert package.deprecation_reason is None
    assert package.deprecation_severity is None
    assert repr(package) == req_repr
    assert str(package) == req_str


@pytest.mark.parametrize("req", [
    ('foobar>=1.2'),
    ('foobar'),
])
def test_package__invalid_req_raises_ValueError(req):
    pytest.raises(
        ValueError,
        Package,
        req,
    )


@pytest.mark.parametrize("package, other, expected", [
    (Package('foobar==1.2'), Package('foobar==1.2'), True),
    (Package('foobar==1.2'), Package('foobar==2.0'), False),
    (Package('foobar==1.2'), None, False),
    (Package('foobar==1.2'), object(), False),
])
def test_package__equals(package, other, expected):
    assert (package == other) is expected


@pytest.mark.parametrize("package, other, expected", [
    (Package('foobar==1.2'), Package('foobar==1.2'), False),
    (Package('foobar==1.2'), Package('foobar==2.0'), True),
    (Package('foobar==1.2'), None, True),
    (Package('foobar==1.2'), object(), True),
])
def test_package__not_equals(package, other, expected):
    assert (package != other) is expected


@pytest.mark.parametrize("package, expected", [
    (Package('foobar==1.2'), 'Package(foobar==1.2)'),
])
def test_package__repr(package, expected):
    assert repr(package) == expected


@pytest.mark.parametrize("package1, package2, expected", [
    (Package('foobar==1.2'), Package('foobar==1.2'), 1),
    (Package('foobar==1.0'), Package('foobar==1.2'), 2),
])
def test_package__hash(package1, package2, expected):
    s = {package1, package2}
    assert len(s) == expected


@pytest.mark.parametrize("package1, package2, expected", [
    (Package('foobar==1.2'), Package('foobar==1.2'), [Package('foobar==1.2'), Package('foobar==1.2')]),  # all equal
    (Package('foobar==1.0'), Package('foobar==1.2'), [Package('foobar==1.0'), Package('foobar==1.2')]),  # already sorted
    (Package('foobar==1.2'), Package('foobar==1.0'), [Package('foobar==1.0'), Package('foobar==1.2')]),
    (Package('foobar==2.0'), Package('foobar==1.0'), [Package('foobar==1.0'), Package('foobar==2.0')]),
    (Package('foobar==2.0'), Package('foobar==0.0'), [Package('foobar==0.0'), Package('foobar==2.0')]),
    (Package('FOOBAR==2.0'), Package('foobar==1.0'), [Package('foobar==1.0'), Package('FOOBAR==2.0')]),  # sort case-insensitive, upper-case comes first by default
    (Package('foo-bar==1.0'), Package('foo==2.0'),   [Package('foo==2.0'), Package('foo-bar==1.0')]),  # same prefix
])
def test_package__lt_for_sorting(package1, package2, expected):
    assert sorted([package1, package2]) == expected


@pytest.mark.parametrize("package, deprecate_kwargs", [
    (Package('foobar==1.0'), {}),
    (Package('foobar==1.0'), {'reason': 'because'}),
    (Package('foobar==1.0'), {'deprecated_by': 'some-deprecator'}),
    (Package('foobar==1.0'), {'severity': 'warn'}),
    (Package('foobar==1.0'), {'reason': 'why not?', 'deprecated_by': 'The Return Of The Deprecator', 'severity': 'warn'}),
])
def test_package__deprecate(package, deprecate_kwargs):
    package.deprecate(**deprecate_kwargs)

    assert package.deprecated is True
    assert package.deprecation_reason == deprecate_kwargs.get('reason')
    assert package.deprecated_by == deprecate_kwargs.get('deprecated_by')
    assert package.deprecation_severity == deprecate_kwargs.get('severity', 'error')



import pytest


@pytest.mark.parametrize("requirements", [
    ({'requirements': [
        {
            'requirement': 'foo==1.0',
        }
    ]}),
    ({'requirements': [
        {
            'requirement': 'foo==1.0',
            'severity': 'error'  # error
        }
    ]}),
    ({'requirements': [
        {
            'requirement': 'foo==1.0',
            'severity': 'warn'  # warn
        }
    ]}),
    ({'requirements': [
        {
            'requirement': 'foo==1.0',
            'reason': 'fix it'
        }
    ]}),
    ({'requirements': [
        {
            'requirement': 'foo==1.0',
            'reason': 'fix it',
            'severity': 'warn'
        }
    ]}),
])
def test_validate__valid_schema(requirements):
    from defrost.validation import validate
    validate(requirements)


@pytest.mark.parametrize("requirements", [
    ({'requirements': [
        {
            'requirement': 1234,  # must be string
        }
    ]}),
    ({'requirements': [
        {
            # no requirement, is required
            # no reason
            # no severity
        }
    ]}),
    ({'requirements': [
        {
            'requirement': 'foo==1.0',
            'severity': 'foobar'  # invalid
        }
    ]}),
    ({'requirements': [
        {
            'requirement': 'foo==1.0',
            'reason': 1234,  # must be string
        }
    ]}),
])
def test_validate__invalid_schema(requirements):
    from defrost.validation import validate
    pytest.raises(
        Exception,
        validate,
        requirements
    )



import pytest

from defrost import Requirement, Package


@pytest.mark.parametrize("req, req_name, req_specs, req_id, req_repr, req_str", [
    ('foobar==1.2', 'foobar', [('==', '1.2')], 'foobar', 'Requirement(foobar==1.2)', 'foobar==1.2'),
    ('foo>=1.2,<2.0', 'foo', [('>=', '1.2'), ('<', '2.0')], 'foo', 'Requirement(foo<2.0,>=1.2)', 'foo<2.0,>=1.2'),
])
def test_requirement(req, req_name, req_specs, req_id, req_repr, req_str):
    from defrost import Requirement
    req = Requirement(req)
    assert req.id == req_id
    assert req.name == req_name
    assert sorted(req.specifiers) == sorted(req_specs)
    assert repr(req) == req_repr
    assert str(req) == req_str


@pytest.mark.parametrize("req1, req2, expected", [
    (Requirement('foobar==1.2'), Requirement('foobar==1.2'), True),
    (Requirement('foobar>=1.2'), Requirement('foobar==1.2'), False),
    (Requirement('foobar==1.2'), Requirement('foobar'), False),
    (Requirement('foobar==1.2'), None, False),
    (Requirement('foobar>=1.2'), object(), False),
])
def test_requirement__equals(req1, req2, expected):
    assert (req1 == req2) is expected


@pytest.mark.parametrize("req1, req2, expected", [
    (Requirement('foobar==1.2'), Requirement('foobar==1.2'), False),
    (Requirement('foobar>=1.2'), Requirement('foobar==1.2'), True),
    (Requirement('foobar==1.2'), Requirement('foobar'), True),
    (Requirement('foobar==1.2'), None, True),
    (Requirement('foobar>=1.2'), object(), True),
])
def test_requirement__not_equals(req1, req2, expected):
    assert (req1 != req2) is expected


@pytest.mark.parametrize("package, req, expected", [
    (Package('foobar==1.2'), Requirement('foobar'), True),
    (Package('foobar==1.2'), Requirement('foobar==1.2'), True),
    (Package('foobar==1.2'), Requirement('foobar>=1.2'), True),
    (Package('foobar==1.2'), Requirement('foobar<=1.2'), True),
    (Package('foobar==1.2'), Requirement('foobar>=1.0,<2.0'), True),
    (Package('foobar==1.2'), Requirement('foobar>2.0'), False),
    (Package('foobar==1.2'), Requirement('foobar<1.0'), False),
    (Package('foobar==1.2'), Requirement('foobar<1.2'), False),
    (Package('foobar==1.2'), Requirement('foobar>1.2'), False),
    (Package('foobar==1.2'), Requirement('foo'), False),
])
def test_requirement__contains_package(package, req, expected):
    assert (package in req) is expected


@pytest.mark.parametrize("req, expected", [
    (Requirement('foobar>=1.2'), 'Requirement(foobar>=1.2)'),
])
def test_requirement__repr(req, expected):
    assert repr(req) == expected


@pytest.mark.parametrize("req1, req2, expected", [
    (Requirement('foobar==1.2'), Requirement('foobar==1.2'), 1),
    (Requirement('foobar==1.2'), Requirement('foobar>=1.2'), 2),
    (Requirement('foobar==1.0'), Requirement('foobar==1.2'), 2),
])
def test_requirement__hash(req1, req2, expected):
    s = {req1, req2}
    assert len(s) == expected



from jsonschema import validate as _validate


REQUIREMENT_ITEM = {
    'title': 'Requirement',
    'description': 'A single requirement item.',
    'type': 'object',
    'additionalProperties': False,
    'properties': {
        'requirement': {'type': 'string'},
        'reason': {'type': 'string'},
        'severity': {'type': 'string', 'enum': ['error', 'warn']},
    },
    'required': ['requirement']
}


SCHEMA = {
    'title': 'Requirement collection',
    'description': 'A collection of requirements.',
    'type': 'object',
    'additionalProperties': False,
    'properties': {
        'requirements': {
            'type': 'array',
            'items': REQUIREMENT_ITEM,
        },
    },
    'required': ['requirements']
}


def validate(requirements):
    _validate(requirements, SCHEMA)



import pytest
from click.testing import CliRunner


@pytest.mark.parametrize("yaml, pipfreeze, output, exit_code", [
    ('tests/reqs.yml', 'tests/pipfreeze_no_matching_req.txt', '', 0),
    ('tests/reqs.yml', 'tests/pipfreeze_satisfied_matching_req.txt', '', 0),
    ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_matching_req.txt', 'error: Package(foo==0.1) does not satisfy Requirement(foo>=1.0): upgrade now!\n', 1),
    ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_reqs_and_contains_links.txt', 'error: Package(foo==0.1) does not satisfy Requirement(foo>=1.0): upgrade now!\n', 1),
    ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_reqs_with_severity.txt', 'warn: Package(testseverity==0.1) does not satisfy Requirement(testseverity>=1.0): upgrade now!\n', 0),
])
def test_defrost__cli(yaml, pipfreeze, output, exit_code):
    from defrost.cli import defrost

    runner = CliRunner()
    result = runner.invoke(
        defrost, [yaml, pipfreeze], catch_exceptions=False
    )
    assert result.output == output
    assert result.exit_code == exit_code


@pytest.mark.parametrize("exit_mode_option, yaml, pipfreeze, exit_code", [
    (exit_mode_option, yaml, pipfreeze, exit_code)
    for yaml, pipfreeze, exit_code in [
        ('tests/reqs.yml', 'tests/pipfreeze_no_matching_req.txt', 0),
        ('tests/reqs.yml', 'tests/pipfreeze_satisfied_matching_req.txt', 0),
        ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_matching_req.txt', 0),
        ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_reqs_with_severity.txt', 0),
    ] for exit_mode_option in ('-x', '--exit-mode')
])
def test_defrost__exit_mode_soft(exit_mode_option, yaml, pipfreeze, exit_code):
    from defrost.cli import defrost

    runner = CliRunner()
    result = runner.invoke(
        defrost, [exit_mode_option, 'soft', yaml, pipfreeze], catch_exceptions=False
    )
    assert result.exit_code == exit_code


@pytest.mark.parametrize("exit_mode_option, yaml, pipfreeze, exit_code", [
    (exit_mode_option, yaml, pipfreeze, exit_code)
    for yaml, pipfreeze, exit_code in [
        ('tests/reqs.yml', 'tests/pipfreeze_no_matching_req.txt', 0),
        ('tests/reqs.yml', 'tests/pipfreeze_satisfied_matching_req.txt', 0),
        ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_matching_req.txt', 1),
        ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_reqs_with_severity.txt', 0),
    ] for exit_mode_option in ('-x', '--exit-mode')
])
def test_defrost__exit_mode_normal(exit_mode_option, yaml, pipfreeze, exit_code):
    from defrost.cli import defrost

    runner = CliRunner()
    result = runner.invoke(
        defrost, [exit_mode_option, 'normal', yaml, pipfreeze], catch_exceptions=False
    )
    assert result.exit_code == exit_code


@pytest.mark.parametrize("exit_mode_option, yaml, pipfreeze, exit_code", [
    (exit_mode_option, yaml, pipfreeze, exit_code)
    for yaml, pipfreeze, exit_code in [
        ('tests/reqs.yml', 'tests/pipfreeze_no_matching_req.txt', 0),
        ('tests/reqs.yml', 'tests/pipfreeze_satisfied_matching_req.txt', 0),
        ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_matching_req.txt', 1),
        ('tests/reqs.yml', 'tests/pipfreeze_unsatisfied_reqs_with_severity.txt', 1),
    ] for exit_mode_option in ('-x', '--exit-mode')
])
def test_defrost__exit_mode_hard(exit_mode_option, yaml, pipfreeze, exit_code):
    from defrost.cli import defrost

    runner = CliRunner()
    result = runner.invoke(
        defrost, [exit_mode_option, 'hard', yaml, pipfreeze], catch_exceptions=False
    )
    assert result.exit_code == exit_code


@pytest.mark.parametrize("yaml, exit_code", [
    ('tests/reqs.yml', 0),
    ('tests/reqs_invalid_data.yml', 1),
    ('tests/reqs_unparsable.yml', 1),
])
def test_defrost_lint(yaml, exit_code):
    from defrost.cli import lint

    runner = CliRunner()
    result = runner.invoke(
        lint, [yaml], catch_exceptions=False
    )
    assert result.exit_code == exit_code



import pytest

from defrost import Package, Requirement


@pytest.mark.parametrize("freeze, expected", [
    ("", False),
    ("foobar==1.2.3", True),
    ("foo==1.2.3\nbar==2.0", True),
])
def test_pip_freeze__bool(freeze, expected):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    assert bool(pip_freeze) is expected


@pytest.mark.parametrize("freeze, expected", [
    ("", []),
    ("foobar==1.2.3", [Package("foobar==1.2.3")]),
    ("foo==1.2.3\nbar==2.0", [Package("foo==1.2.3"), Package("bar==2.0")]),
    ("-f /opt/pip/wheels\n-e git+git@github.com:SurveyMonkey/defrost.git\nfoo==1.2.3\n## !! Could not determine repository location\nbar==2.0", [Package("foo==1.2.3"), Package("bar==2.0")]),
    ("## FIXME: could not find svn URL in dependency_links for this package:\ndeveloperweb===3-update-etcdev-r0", [Package("developerweb===3-update-etcdev-r0")]),
    ("-i https://pypi.python.org/simple\nfoo==1.2.3", [Package("foo==1.2.3")]),  # test -i
    ("foo==1.2.3\n\nbar==2.0\n\n", [Package("foo==1.2.3"), Package("bar==2.0")]),  # test blank lines
    (b"foo==1.2.3\nbar==2.0", [Package("foo==1.2.3"), Package("bar==2.0")]),  # test pip freeze as bytes
])
def test_pip_freeze__iter(freeze, expected):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    assert list(pip_freeze) == expected


@pytest.mark.parametrize("freeze, expected", [
    ("", 0),
    ("foobar==1.2.3", 1),
    ("foo==1.2.3\nbar==2.0", 2),
])
def test_pip_freeze__len(freeze, expected):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    assert len(pip_freeze) == expected


@pytest.mark.parametrize("freeze, package, expected", [
    ('foo==1.2', 'foo', True),
    ('foo==1.2', 'FOO', True),
    ('foo==1.2', 'foo==1.2', True),
    ('foo==1.2', 'foo>=1.0', True),
    ('foo==1.2', 'foo<2.0', True),
    ("foobar==1.2", 'foobar>=1.0,<2.0', True),
    ('foo==1.2', Requirement('foo'), True),
    ('foo==1.2', Requirement('foo==1.2'), True),
    ('foo==1.2', Requirement('foo>=1.0'), True),
    ('foo==1.2', Package('foo==1.2'), True),
    ('foo==1.2\nbar==1.0', 'bar', True),
    ("foo==1.2\nbar==2.0", 'foobar', False),
    ('foo==1.2', 'bar', False),
    ('foo==1.2', 'foo==1.3', False),
    ('foo==1.2', Requirement('bar'), False),
    ('foo==1.2', Requirement('foo==1.3'), False),
    ('foo==1.2', Requirement('foo>=2.0'), False),
    ('foo==1.2', Requirement('bar==1.2'), False),
    ('foo==1.2', Package('bar==1.2'), False),
    ("", "foo==1.2", False),
])
def test_pip_freeze__contains(freeze, package, expected):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    assert (package in pip_freeze) is expected


@pytest.mark.parametrize("freeze, package, expected", [
    ('foo==1.2', 'foo', Package('foo==1.2')),
    ('foo==1.2', 'FOO', Package('foo==1.2')),
    ('foo==1.2\nbar==1.0', 'bar', Package('bar==1.0')),
])
def test_pip_freeze__getitems_found(freeze, package, expected):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    assert pip_freeze[package] == expected


@pytest.mark.parametrize("freeze, package", [
    ('foo==1.2', 'bar'),
])
def test_pip_freeze__getitems_KeyError(freeze, package):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    pytest.raises(KeyError, pip_freeze.__getitem__, package)


@pytest.mark.parametrize("freeze, package, expected", [
    ('foo==1.2', 'bar', None),
    ('foo==1.2', 'foo', Package('foo==1.2')),
    ('foo==1.2', 'FOO', Package('foo==1.2')),
])
def test_pip_freeze__get_without_default(freeze, package, expected):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    assert pip_freeze.get(package) == expected


@pytest.mark.parametrize("freeze, package, default, expected", [
    ('foo==1.2', 'bar', None, None),
    ('foo==1.2', 'bar', 'some-default', 'some-default'),
    ('foo==1.2', 'foo', 'some-default', Package('foo==1.2')),
    ('foo==1.2', 'FOO', 'some-default', Package('foo==1.2')),
])
def test_pip_freeze__get_with_default(freeze, package, default, expected):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    assert pip_freeze.get(package, default) == expected


@pytest.mark.parametrize("freeze, reqs, expected_deprecated", [
    ("foo==1.2", {'requirements': [{'requirement': 'foo', 'reason': 'hello'}]}, []),
    ("foo==1.2", {'requirements': [{'requirement': 'foo<1.0', 'reason': 'upgrade'}]}, [('foo==1.2', 'upgrade', 'error')]),
    ("foo==1.2", {'requirements': [{'requirement': 'foo>=1.0', 'reason': 'upgrade'}]}, []),
    ("foo==1.2\nbar==2.0", {'requirements': [
        {'requirement': 'foo>=1.0', 'reason': 'upgrade'},
        {'requirement': 'bar<2.0', 'reason': 'downgrade'}]}, [('bar==2.0', 'downgrade', 'error')]),
    ("foo==1.2", {'requirements': []}, []),
    ("foo==1.2", {'requirements': [{'requirement': 'bar>=1.0', 'reason': 'upgrade'}]}, []),
    ("foo==1.2", {'requirements': [{'requirement': 'foo<1.0', 'reason': 'upgrade', 'severity': 'warn'}]}, [('foo==1.2', 'upgrade', 'warn')]),
    ("foo==1.2", {'requirements': [{'requirement': 'foo<1.0', 'severity': 'warn'}]}, [('foo==1.2', None, 'warn')]),
])
def test_pip_freeze__load_requirements(freeze, reqs, expected_deprecated):
    from defrost import PipFreeze
    pip_freeze = PipFreeze(freeze)
    pip_freeze.load_requirements(reqs)
    assert len(pip_freeze.deprecated) == len(expected_deprecated)
    for package, (pin, reason, severity) in zip(pip_freeze.deprecated, expected_deprecated):
        assert package.deprecated is True
        assert str(package) == pin
        assert package.deprecation_reason == reason
        assert package.deprecation_severity == severity



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

version = '0.4.0'

README = read('README.rst')
CHANGES = read('CHANGES.rst')

requires = [
    'PyYAML',
    'click>=4.0',
    'jsonschema',
]

setup(
    name="defrost",
    version=version,
    author="Alex Conrad",
    author_email="alexandre@surveymonkey.com",
    maintainer="Alex Conrad",
    maintainer_email="alexandre@surveymonkey.com",
    description="A tool to audit pip freeze outputs "
                "and test version requirements",
    license="MIT License",
    keywords="defrost pip freeze output audit version requirements deprecate",
    url="https://github.com/SurveyMonkey/defrost",
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['tests']),
    long_description='%s\n\n%s' % (README, CHANGES),
    setup_requires=['setuptools_git'],
    install_requires=requires,
    classifiers=[
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4"
    ],
    entry_points={
        'console_scripts': [
            'defrost=defrost.cli:defrost',
            'defrost-lint=defrost.cli:lint',
        ],
    },
)
